library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity musiccontroller is
port (

    clk        : in  std_logic;
    reset_n    : in  std_logic;
    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  std_logic_vector(16 downto 0);
    readdata   : out std_logic_vector(15 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);
    irq        : out std_logic;
	 

	AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
	AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
	AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
	AUD_DACDAT   : out  std_logic;   --    Audio CODEC DAC Data
	AUD_BCLK     : inout std_logic --    Audio CODEC Bit-Stream Clock
  );
  
  end musiccontroller;
  
  architecture mc of musiccontroller is
  
  signal audio_request : std_logic;
  signal audio_clock : unsigned(1 downto 0) := "00";
  signal store : std_logic;
  signal clk_25:std_logic;
  type buffer_type is array (0 to 127) of std_logic_vector (15 downto 0);
  signal buffer1 : buffer_type;
  signal data    : std_logic_vector(15 downto 0);
  signal counter : unsigned(6 downto 0);
  signal ram_address : unsigned(16 downto 0);
  

  
   component de2_wm8731_audio is port(
		    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    		 reset_n : in std_logic;
		    test_mode : in std_logic;       --    Audio CODEC controller test mode
		    audio_request : out std_logic;  --    Audio controller request new data
		    data       : in std_logic_vector(15 downto 0);
			 address    : in  std_logic_vector(16 downto 0);
			 write      : in std_logic;
			 chipselect : in std_logic;
			 counter_out:  out unsigned (6 downto 0); 
  
		    -- Audio interface signals
    		 AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
		    AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
		    AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
		    AUD_DACDAT   : out  std_logic;   --    Audio CODEC DAC Data
		    AUD_BCLK     : inout std_logic--      Audio CODEC Bit-Stream Clock

		 );
		 end component de2_wm8731_audio;
		 
		 
		 
--		 component de2_audio_pll is
--PORT
--(
--inclk0		: IN STD_LOGIC  := '0';
--c0		: OUT STD_LOGIC 
--);end component de2_audio_pll;



--	process (clk)
--	begin
--		if rising_edge(clk) then
--			if reset_n = '0' then
--				  store <= '0';
--			else
--				if chipSelect = '1' and write = '1'  then
--					store <= '1';				
--				end if;	5
--			end if;
--		end if;
--	end process;
begin

ram_address <= unsigned(address(16 downto 0));

V1: de2_wm8731_audio port map (
    clk => clk_25,
    reset_n   => '1',
    test_mode => '0',         
    audio_request => audio_request,
    data =>  data ,
	 address => address,
	 write   => write,
	 chipselect => chipselect,
	 counter_out=> counter,
  
    -- Audio interface signals
    AUD_ADCLRCK  => AUD_ADCLRCK,
    AUD_ADCDAT   => AUD_ADCDAT,
    AUD_DACLRCK  => AUD_DACLRCK,
    AUD_DACDAT   => AUD_DACDAT,
    AUD_BCLK     => AUD_BCLK
  );
  
  
--  V2: de2_audio_pll port map(
--  inclk0 => clk,
--  c0     => clk_18
--  );
  
 process (clk)
  begin
    if rising_edge(clk) then
	   clk_25 <= not clk_25;
      if reset_n = '0' then
        irq <= '0';
      else
        if counter = "1111111"  then
          irq <= '1';
        elsif write = '1' and chipselect = '1' then
          irq <= '0';  -- important to reset the irq
        end if;
      end if;
    end if;
  end process;
  
  process (clk) begin
	if rising_edge (clk) then
		data <= buffer1(to_integer(counter));
	if chipselect = '1' and write = '1'  then
	   buffer1 (to_integer (ram_address)) <= writedata;
		end if;
	end if;
end process;
  
  end architecture;
  